pub type FuseOperations = fuse_operations;Aliased Type
#[repr(C)]pub struct FuseOperations {Show 58 fields
pub getattr: Option<unsafe extern "C" fn(*const i8, *mut stat) -> i32>,
pub readlink: Option<unsafe extern "C" fn(*const i8, *mut i8, usize) -> i32>,
pub getdir: Option<unsafe extern "C" fn(*const i8, *mut fuse_dirhandle, Option<unsafe extern "C" fn(*mut fuse_dirhandle, *const i8, i32, u64) -> i32>) -> i32>,
pub mknod: Option<unsafe extern "C" fn(*const i8, u32, u64) -> i32>,
pub mkdir: Option<unsafe extern "C" fn(*const i8, u32) -> i32>,
pub unlink: Option<unsafe extern "C" fn(*const i8) -> i32>,
pub rmdir: Option<unsafe extern "C" fn(*const i8) -> i32>,
pub symlink: Option<unsafe extern "C" fn(*const i8, *const i8) -> i32>,
pub rename: Option<unsafe extern "C" fn(*const i8, *const i8) -> i32>,
pub link: Option<unsafe extern "C" fn(*const i8, *const i8) -> i32>,
pub chmod: Option<unsafe extern "C" fn(*const i8, u32) -> i32>,
pub chown: Option<unsafe extern "C" fn(*const i8, u32, u32) -> i32>,
pub truncate: Option<unsafe extern "C" fn(*const i8, i64) -> i32>,
pub utime: Option<unsafe extern "C" fn(*const i8, *mut utimbuf) -> i32>,
pub open: Option<unsafe extern "C" fn(*const i8, *mut fuse_file_info) -> i32>,
pub read: Option<unsafe extern "C" fn(*const i8, *mut i8, usize, i64, *mut fuse_file_info) -> i32>,
pub write: Option<unsafe extern "C" fn(*const i8, *const i8, usize, i64, *mut fuse_file_info) -> i32>,
pub statfs: Option<unsafe extern "C" fn(*const i8, *mut statvfs) -> i32>,
pub flush: Option<unsafe extern "C" fn(*const i8, *mut fuse_file_info) -> i32>,
pub release: Option<unsafe extern "C" fn(*const i8, *mut fuse_file_info) -> i32>,
pub fsync: Option<unsafe extern "C" fn(*const i8, i32, *mut fuse_file_info) -> i32>,
pub setxattr: Option<unsafe extern "C" fn(*const i8, *const i8, *const i8, usize, i32, u32) -> i32>,
pub getxattr: Option<unsafe extern "C" fn(*const i8, *const i8, *mut i8, usize, u32) -> i32>,
pub listxattr: Option<unsafe extern "C" fn(*const i8, *mut i8, usize) -> i32>,
pub removexattr: Option<unsafe extern "C" fn(*const i8, *const i8) -> i32>,
pub opendir: Option<unsafe extern "C" fn(*const i8, *mut fuse_file_info) -> i32>,
pub readdir: Option<unsafe extern "C" fn(*const i8, *mut c_void, Option<unsafe extern "C" fn(*mut c_void, *const i8, *const stat, i64) -> i32>, i64, *mut fuse_file_info) -> i32>,
pub releasedir: Option<unsafe extern "C" fn(*const i8, *mut fuse_file_info) -> i32>,
pub fsyncdir: Option<unsafe extern "C" fn(*const i8, i32, *mut fuse_file_info) -> i32>,
pub init: Option<unsafe extern "C" fn(*mut fuse_conn_info) -> *mut c_void>,
pub destroy: Option<unsafe extern "C" fn(*mut c_void)>,
pub access: Option<unsafe extern "C" fn(*const i8, i32) -> i32>,
pub create: Option<unsafe extern "C" fn(*const i8, u32, *mut fuse_file_info) -> i32>,
pub ftruncate: Option<unsafe extern "C" fn(*const i8, i64, *mut fuse_file_info) -> i32>,
pub fgetattr: Option<unsafe extern "C" fn(*const i8, *mut stat, *mut fuse_file_info) -> i32>,
pub lock: Option<unsafe extern "C" fn(*const i8, *mut fuse_file_info, i32, *mut flock) -> i32>,
pub utimens: Option<unsafe extern "C" fn(*const i8, *const timespec) -> i32>,
pub bmap: Option<unsafe extern "C" fn(*const i8, usize, *mut u64) -> i32>,
pub reserved00: Option<unsafe extern "C" fn(*mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void) -> i32>,
pub reserved01: Option<unsafe extern "C" fn(*mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void) -> i32>,
pub reserved02: Option<unsafe extern "C" fn(*mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void) -> i32>,
pub reserved03: Option<unsafe extern "C" fn(*mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void) -> i32>,
pub reserved04: Option<unsafe extern "C" fn(*mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void) -> i32>,
pub reserved05: Option<unsafe extern "C" fn(*mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void) -> i32>,
pub reserved06: Option<unsafe extern "C" fn(*mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void) -> i32>,
pub reserved07: Option<unsafe extern "C" fn(*mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void) -> i32>,
pub reserved08: Option<unsafe extern "C" fn(*mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void) -> i32>,
pub reserved09: Option<unsafe extern "C" fn(*mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void) -> i32>,
pub reserved10: Option<unsafe extern "C" fn(*mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void) -> i32>,
pub setvolname: Option<unsafe extern "C" fn(*const i8) -> i32>,
pub exchange: Option<unsafe extern "C" fn(*const i8, *const i8, u64) -> i32>,
pub getxtimes: Option<unsafe extern "C" fn(*const i8, *mut timespec, *mut timespec) -> i32>,
pub setbkuptime: Option<unsafe extern "C" fn(*const i8, *const timespec) -> i32>,
pub setchgtime: Option<unsafe extern "C" fn(*const i8, *const timespec) -> i32>,
pub setcrtime: Option<unsafe extern "C" fn(*const i8, *const timespec) -> i32>,
pub chflags: Option<unsafe extern "C" fn(*const i8, u32) -> i32>,
pub setattr_x: Option<unsafe extern "C" fn(*const i8, *mut setattr_x) -> i32>,
pub fsetattr_x: Option<unsafe extern "C" fn(*const i8, *mut setattr_x, *mut fuse_file_info) -> i32>,
}Fields
getattr: Option<unsafe extern "C" fn(*const i8, *mut stat) -> i32>Get file attributes.
Similar to stat(). The ‘st_dev’ and ‘st_blksize’ fields are ignored. The ‘st_ino’ field is ignored except if the ‘use_ino’ mount option is given.
readlink: Option<unsafe extern "C" fn(*const i8, *mut i8, usize) -> i32>Read the target of a symbolic link
The buffer should be filled with a null terminated string. The buffer size argument includes the space for the terminating null character. If the linkname is too long to fit in the buffer, it should be truncated. The return value should be 0 for success.
getdir: Option<unsafe extern "C" fn(*const i8, *mut fuse_dirhandle, Option<unsafe extern "C" fn(*mut fuse_dirhandle, *const i8, i32, u64) -> i32>) -> i32>mknod: Option<unsafe extern "C" fn(*const i8, u32, u64) -> i32>Create a file node
This is called for creation of all non-directory, non-symlink nodes. If the filesystem defines a create() method, then for regular files that will be called instead.
mkdir: Option<unsafe extern "C" fn(*const i8, u32) -> i32>Create a directory
unlink: Option<unsafe extern "C" fn(*const i8) -> i32>Remove a file
rmdir: Option<unsafe extern "C" fn(*const i8) -> i32>Remove a directory
symlink: Option<unsafe extern "C" fn(*const i8, *const i8) -> i32>Create a symbolic link
rename: Option<unsafe extern "C" fn(*const i8, *const i8) -> i32>Rename a file
link: Option<unsafe extern "C" fn(*const i8, *const i8) -> i32>Create a hard link to a file
chmod: Option<unsafe extern "C" fn(*const i8, u32) -> i32>Change the permission bits of a file
chown: Option<unsafe extern "C" fn(*const i8, u32, u32) -> i32>Change the owner and group of a file
truncate: Option<unsafe extern "C" fn(*const i8, i64) -> i32>Change the size of a file
utime: Option<unsafe extern "C" fn(*const i8, *mut utimbuf) -> i32>Change the access and/or modification times of a file
Deprecated, use utimens() instead.
open: Option<unsafe extern "C" fn(*const i8, *mut fuse_file_info) -> i32>File open operation
No creation, or truncation flags (O_CREAT, O_EXCL, O_TRUNC) will be passed to open(). Open should check if the operation is permitted for the given flags. Optionally open may also return an arbitrary filehandle in the fuse_file_info structure, which will be passed to all file operations.
Changed in version 2.2
read: Option<unsafe extern "C" fn(*const i8, *mut i8, usize, i64, *mut fuse_file_info) -> i32>Read data from an open file
Read should return exactly the number of bytes requested except on EOF or error, otherwise the rest of the data will be substituted with zeroes. An exception to this is when the ‘direct_io’ mount option is specified, in which case the return value of the read system call will reflect the return value of this operation.
Changed in version 2.2
write: Option<unsafe extern "C" fn(*const i8, *const i8, usize, i64, *mut fuse_file_info) -> i32>Write data to an open file
Write should return exactly the number of bytes requested except on error. An exception to this is when the ‘direct_io’ mount option is specified (see read operation).
Changed in version 2.2
statfs: Option<unsafe extern "C" fn(*const i8, *mut statvfs) -> i32>Get file system statistics
The ‘f_frsize’, ‘f_favail’, ‘f_fsid’ and ‘f_flag’ fields are ignored
Replaced ‘struct statfs’ parameter with ‘struct statvfs’ in version 2.5
flush: Option<unsafe extern "C" fn(*const i8, *mut fuse_file_info) -> i32>Possibly flush cached data
BIG NOTE: This is not equivalent to fsync(). It’s not a request to sync dirty data.
Flush is called on each close() of a file descriptor. So if a filesystem wants to return write errors in close() and the file has cached dirty data, this is a good place to write back data and return any errors. Since many applications ignore close() errors this is not always useful.
NOTE: The flush() method may be called more than once for each open(). This happens if more than one file descriptor refers to an opened file due to dup(), dup2() or fork() calls. It is not possible to determine if a flush is final, so each flush should be treated equally. Multiple write-flush sequences are relatively rare, so this shouldn’t be a problem.
Filesystems shouldn’t assume that flush will always be called after some writes, or that if will be called at all.
Changed in version 2.2
release: Option<unsafe extern "C" fn(*const i8, *mut fuse_file_info) -> i32>Release an open file
Release is called when there are no more references to an open file: all file descriptors are closed and all memory mappings are unmapped.
For every open() call there will be exactly one release() call with the same flags and file descriptor. It is possible to have a file opened more than once, in which case only the last release will mean, that no more reads/writes will happen on the file. The return value of release is ignored.
Changed in version 2.2
fsync: Option<unsafe extern "C" fn(*const i8, i32, *mut fuse_file_info) -> i32>Synchronize file contents
If the datasync parameter is non-zero, then only the user data should be flushed, not the meta data.
Changed in version 2.2
setxattr: Option<unsafe extern "C" fn(*const i8, *const i8, *const i8, usize, i32, u32) -> i32>getxattr: Option<unsafe extern "C" fn(*const i8, *const i8, *mut i8, usize, u32) -> i32>listxattr: Option<unsafe extern "C" fn(*const i8, *mut i8, usize) -> i32>List extended attributes
removexattr: Option<unsafe extern "C" fn(*const i8, *const i8) -> i32>Remove extended attributes
opendir: Option<unsafe extern "C" fn(*const i8, *mut fuse_file_info) -> i32>Open directory
This method should check if the open operation is permitted for this directory
Introduced in version 2.3
readdir: Option<unsafe extern "C" fn(*const i8, *mut c_void, Option<unsafe extern "C" fn(*mut c_void, *const i8, *const stat, i64) -> i32>, i64, *mut fuse_file_info) -> i32>Read directory
This supersedes the old getdir() interface. New applications should use this.
The filesystem may choose between two modes of operation:
-
The readdir implementation ignores the offset parameter, and passes zero to the filler function’s offset. The filler function will not return ‘1’ (unless an error happens), so the whole directory is read in a single readdir operation. This works just like the old getdir() method.
-
The readdir implementation keeps track of the offsets of the directory entries. It uses the offset parameter and always passes non-zero offset to the filler function. When the buffer is full (or an error happens) the filler function will return ‘1’.
Introduced in version 2.3
releasedir: Option<unsafe extern "C" fn(*const i8, *mut fuse_file_info) -> i32>Release directory
Introduced in version 2.3
fsyncdir: Option<unsafe extern "C" fn(*const i8, i32, *mut fuse_file_info) -> i32>Synchronize directory contents
If the datasync parameter is non-zero, then only the user data should be flushed, not the meta data
Introduced in version 2.3
init: Option<unsafe extern "C" fn(*mut fuse_conn_info) -> *mut c_void>Initialize filesystem
The return value will passed in the private_data field of fuse_context to all file operations and as a parameter to the destroy() method.
Introduced in version 2.3 Changed in version 2.6
destroy: Option<unsafe extern "C" fn(*mut c_void)>Clean up filesystem
Called on filesystem exit.
Introduced in version 2.3
access: Option<unsafe extern "C" fn(*const i8, i32) -> i32>Check file access permissions
This will be called for the access() system call. If the ‘default_permissions’ mount option is given, this method is not called.
This method is not called under Linux kernel versions 2.4.x
Introduced in version 2.5
create: Option<unsafe extern "C" fn(*const i8, u32, *mut fuse_file_info) -> i32>Create and open a file
If the file does not exist, first create it with the specified mode, and then open it.
If this method is not implemented or under Linux kernel versions earlier than 2.6.15, the mknod() and open() methods will be called instead.
Introduced in version 2.5
ftruncate: Option<unsafe extern "C" fn(*const i8, i64, *mut fuse_file_info) -> i32>Change the size of an open file
This method is called instead of the truncate() method if the truncation was invoked from an ftruncate() system call.
If this method is not implemented or under Linux kernel versions earlier than 2.6.15, the truncate() method will be called instead.
Introduced in version 2.5
fgetattr: Option<unsafe extern "C" fn(*const i8, *mut stat, *mut fuse_file_info) -> i32>Get attributes from an open file
This method is called instead of the getattr() method if the file information is available.
Currently this is only called after the create() method if that is implemented (see above). Later it may be called for invocations of fstat() too.
Introduced in version 2.5
lock: Option<unsafe extern "C" fn(*const i8, *mut fuse_file_info, i32, *mut flock) -> i32>Perform POSIX file locking operation
The cmd argument will be either F_GETLK, F_SETLK or F_SETLKW.
For the meaning of fields in ‘struct flock’ see the man page for fcntl(2). The l_whence field will always be set to SEEK_SET.
For checking lock ownership, the ‘fuse_file_info->owner’ argument must be used.
For F_GETLK operation, the library will first check currently held locks, and if a conflicting lock is found it will return information without calling this method. This ensures, that for local locks the l_pid field is correctly filled in. The results may not be accurate in case of race conditions and in the presence of hard links, but it’s unlikly that an application would rely on accurate GETLK results in these cases. If a conflicting lock is not found, this method will be called, and the filesystem may fill out l_pid by a meaningful value, or it may leave this field zero.
For F_SETLK and F_SETLKW the l_pid field will be set to the pid of the process performing the locking operation.
Note: if this method is not implemented, the kernel will still allow file locking to work locally. Hence it is only interesting for network filesystems and similar.
Introduced in version 2.6
utimens: Option<unsafe extern "C" fn(*const i8, *const timespec) -> i32>Change the access and modification times of a file with nanosecond resolution
Introduced in version 2.6
bmap: Option<unsafe extern "C" fn(*const i8, usize, *mut u64) -> i32>Map block index within file to block index within device
Note: This makes sense only for block device backed filesystems mounted with the ‘blkdev’ option
Introduced in version 2.6
reserved00: Option<unsafe extern "C" fn(*mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void) -> i32>reserved01: Option<unsafe extern "C" fn(*mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void) -> i32>reserved02: Option<unsafe extern "C" fn(*mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void) -> i32>reserved03: Option<unsafe extern "C" fn(*mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void) -> i32>reserved04: Option<unsafe extern "C" fn(*mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void) -> i32>reserved05: Option<unsafe extern "C" fn(*mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void) -> i32>reserved06: Option<unsafe extern "C" fn(*mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void) -> i32>reserved07: Option<unsafe extern "C" fn(*mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void) -> i32>reserved08: Option<unsafe extern "C" fn(*mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void) -> i32>reserved09: Option<unsafe extern "C" fn(*mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void) -> i32>reserved10: Option<unsafe extern "C" fn(*mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void, *mut c_void) -> i32>setvolname: Option<unsafe extern "C" fn(*const i8) -> i32>exchange: Option<unsafe extern "C" fn(*const i8, *const i8, u64) -> i32>getxtimes: Option<unsafe extern "C" fn(*const i8, *mut timespec, *mut timespec) -> i32>setbkuptime: Option<unsafe extern "C" fn(*const i8, *const timespec) -> i32>setchgtime: Option<unsafe extern "C" fn(*const i8, *const timespec) -> i32>setcrtime: Option<unsafe extern "C" fn(*const i8, *const timespec) -> i32>chflags: Option<unsafe extern "C" fn(*const i8, u32) -> i32>setattr_x: Option<unsafe extern "C" fn(*const i8, *mut setattr_x) -> i32>fsetattr_x: Option<unsafe extern "C" fn(*const i8, *mut setattr_x, *mut fuse_file_info) -> i32>